@@ -42,8 +42,6 @@ except ImportError:  # pragma: no cover
 PY2 = sys.version_info[0] == 2
 PY3 = sys.version_info[0] == 3
 PYPY = platform.python_implementation() == 'PyPy'
-NO_ARGSPEC = sys.version_info[:2] >= (3, 5)
-HAS_SIGNATURE = sys.version_info[:2] >= (3, 3)
 CAN_UNPACK_BYTE_ARRAY = sys.version_info[:3] >= (2, 7, 4)
 
 WINDOWS = platform.system() == 'Windows'
@@ -263,54 +261,32 @@ def qualname(f):
         return f.__name__
 
 
-FakeArgSpec = namedtuple(
-    'ArgSpec', ('args', 'varargs', 'keywords', 'defaults'))
-
-
-def signature_argspec(f):
-    from inspect import signature, Parameter, _empty
-    try:
-        if NO_ARGSPEC:
-            sig = signature(f, follow_wrapped=False)
-        else:
-            sig = signature(f)
-    except ValueError:
-        raise TypeError('unsupported callable')
-    args = list(
-        k
-        for k, v in sig.parameters.items()
-        if v.kind in (
-            Parameter.POSITIONAL_ONLY, Parameter.POSITIONAL_OR_KEYWORD))
-    varargs = None
-    keywords = None
-    for k, v in sig.parameters.items():
-        if v.kind == Parameter.VAR_POSITIONAL:
-            varargs = k
-        elif v.kind == Parameter.VAR_KEYWORD:
-            keywords = k
-    defaults = []
-    for a in reversed(args):
-        default = sig.parameters[a].default
-        if default is _empty:
-            break
-        else:
-            defaults.append(default)
-    if defaults:
-        defaults = tuple(reversed(defaults))
-    else:
-        defaults = None
-    return FakeArgSpec(args, varargs, keywords, defaults)
-
-
-if NO_ARGSPEC:
-    ArgSpec = FakeArgSpec
-else:
-    from inspect import ArgSpec
-
 if PY2:
-    from inspect import getargspec
+    FullArgSpec = namedtuple('FullArgSpec', 'args, varargs, varkw, defaults, '
+                             'kwonlyargs, kwonlydefaults, annotations')
+
+    def getfullargspec(func):
+        import inspect
+        args, varargs, varkw, defaults = inspect.getargspec(func)
+        return FullArgSpec(args, varargs, varkw, defaults, [], None, {})
 else:
-    getargspec = signature_argspec
+    from inspect import getfullargspec, FullArgSpec
+
+    if sys.version_info[:2] == (3, 5):
+        # silence deprecation warnings on Python 3.5
+        # (un-deprecated in 3.6 to allow single-source 2/3 code like this)
+        def silence_warnings(func):
+            import warnings
+            import functools
+
+            @functools.wraps(func)
+            def inner(*args, **kwargs):
+                with warnings.catch_warnings():
+                    warnings.simplefilter('ignore', DeprecationWarning)
+                    return func(*args, **kwargs)
+            return inner
+
+        getfullargspec = silence_warnings(getfullargspec)
 
 
 importlib_invalidate_caches = getattr(
@@ -502,3 +478,10 @@ else:
     # the x flag because it's Python 2, so there are no places where this can
     # be thrown.
     FileExistsError = None
+
+
+if PY2:
+    def floor(x):
+        return int(math.floor(x))
+else:
+    floor = math.floor
