@@ -28,8 +28,8 @@ from hypothesis.reporting import debug_report
 from hypothesis.internal.compat import EMPTY_BYTES, Counter, hbytes, \
     hrange, text_type, bytes_from_list, to_bytes_sequence, \
     unicode_safe_repr
-from hypothesis.internal.conjecture.data import Status, StopTest, \
-    ConjectureData
+from hypothesis.internal.conjecture.data import MAX_DEPTH, Status, \
+    StopTest, ConjectureData
 from hypothesis.internal.conjecture.minimizer import minimize
 
 
@@ -83,10 +83,15 @@ class ConjectureRunner(object):
 
     def new_buffer(self):
         assert not self.__tree_is_exhausted()
+
+        def draw_bytes(data, n, distribution):
+            return self.__rewrite_for_novelty(
+                data, self.__zero_bound(data, distribution(self.random, n))
+            )
+
         self.last_data = ConjectureData(
             max_length=self.settings.buffer_size,
-            draw_bytes=lambda data, n, distribution:
-            self.__rewrite_for_novelty(data, distribution(self.random, n))
+            draw_bytes=draw_bytes
         )
         self.test_function(self.last_data)
         self.last_data.freeze()
@@ -283,6 +288,9 @@ class ConjectureRunner(object):
         def draw_zero(data, n, distribution):
             return hbytes(b'\0' * n)
 
+        def draw_max(data, n, distribution):
+            return hbytes([255]) * n
+
         def draw_constant(data, n, distribution):
             return bytes_from_list([
                 self.random.randint(0, 255)
@@ -292,7 +300,9 @@ class ConjectureRunner(object):
             draw_new,
             reuse_existing, reuse_existing,
             draw_existing, draw_smaller, draw_larger,
-            flip_bit, draw_zero, draw_constant,
+            flip_bit,
+            draw_zero, draw_max, draw_zero, draw_max,
+            draw_constant,
         ]
 
         bits = [
@@ -307,11 +317,45 @@ class ConjectureRunner(object):
             else:
                 result = self.random.choice(bits)(data, n, distribution)
 
-            return self.__rewrite_for_novelty(data, result)
+            return self.__rewrite_for_novelty(
+                data, self.__zero_bound(data, result))
 
         return draw_mutated
 
+    def __rewrite(self, data, result):
+        return self.__rewrite_for_novelty(
+            data, self.__zero_bound(data, result)
+        )
+
+    def __zero_bound(self, data, result):
+        """This tries to get the size of the generated data under control by
+        replacing the result with zero if we are too deep or have already
+        generated too much data.
+
+        This causes us to enter "shrinking mode" there and thus reduce
+        the size of the generated data.
+
+        """
+        if (
+            data.depth * 2 >= MAX_DEPTH or
+            (data.index + len(result)) * 2 >= self.settings.buffer_size
+        ):
+            if any(result):
+                data.hit_zero_bound = True
+            return hbytes(len(result))
+        else:
+            return result
+
     def __rewrite_for_novelty(self, data, result):
+        """Take a block that is about to be added to data as the result of a
+        draw_bytes call and rewrite it a small amount to ensure that the result
+        will be novel: that is, not hit a part of the tree that we have fully
+        explored.
+
+        This is mostly useful for test functions which draw a small
+        number of blocks.
+
+        """
         assert isinstance(result, hbytes)
         try:
             node_index = data.__current_node_index
@@ -416,6 +460,9 @@ class ConjectureRunner(object):
                 self.new_buffer()
 
             mutator = self._new_mutator()
+
+            zero_bound_queue = []
+
             while (
                 self.last_data.status != Status.INTERESTING and
                 not self.__tree_is_exhausted()
@@ -434,9 +481,41 @@ class ConjectureRunner(object):
                 ):
                     self.exit_reason = ExitReason.timeout
                     return
-                if mutations >= self.settings.max_mutations:
+                if zero_bound_queue:
+                    # Whenever we generated an example and it hits a bound
+                    # which forces zero blocks into it, this creates a weird
+                    # distortion effect by making certain parts of the data
+                    # stream (especially ones to the right) much more likely
+                    # to be zero. We fix this by redistributing the generated
+                    # data by shuffling it randomly. This results in the
+                    # zero data being spread evenly throughout the buffer.
+                    # Hopefully the shrinking this causes will cause us to
+                    # naturally fail to hit the bound.
+                    # If it doesn't then we will queue the new version up again
+                    # (now with more zeros) and try again.
+                    overdrawn = zero_bound_queue.pop()
+                    buffer = bytearray(overdrawn.buffer)
+                    self.random.shuffle(buffer)
+                    buffer = hbytes(buffer)
+
+                    if buffer == overdrawn.buffer:
+                        continue
+
+                    def draw_bytes(data, n, distribution):
+                        result = buffer[data.index:data.index + n]
+                        if len(result) < n:
+                            result += hbytes(n - len(result))
+                        return self.__rewrite(data, result)
+
+                    data = ConjectureData(
+                        draw_bytes=draw_bytes,
+                        max_length=self.settings.buffer_size,
+                    )
+                    self.test_function(data)
+                    data.freeze()
+                elif mutations >= self.settings.max_mutations:
                     mutations = 0
-                    self.new_buffer()
+                    data = self.new_buffer()
                     mutator = self._new_mutator()
                 else:
                     data = ConjectureData(
@@ -452,7 +531,8 @@ class ConjectureRunner(object):
                             mutations = 0
                     else:
                         mutator = self._new_mutator()
-
+                if getattr(data, 'hit_zero_bound', False):
+                    zero_bound_queue.append(data)
                 mutations += 1
 
         if self.__tree_is_exhausted():
@@ -479,6 +559,85 @@ class ConjectureRunner(object):
             self.exit_reason = ExitReason.flaky
             return
 
+        self.shrink()
+
+    def zero_blocks(self):
+        """Try replacing blocks with zero blocks, starting from the right and
+        proceeding leftwards.
+
+        Normally we would proceed from left to right, in keeping with
+        our policy of lexicographic minimization - making shrinks to the
+        right seems like it should be "wasted work" which we might undo
+        later.
+
+        The motivation for doing it this way is that this can unlock
+        shrinks that would become impossible otherwise: If we shrink
+        entirely moving rightwards, then this ends up with a lot of the
+        complexity of an example "trapped at the end", leaving a lot of
+        dead space in the middle. An example of where this can happen is
+        with lists or matrices defined by a length parameter, where only
+        one or two of the values actually matter: If we start from the
+        left then what we'll find is we replace all the early values with
+        zero, leave the later values as the ones that matter, and then we
+        can't shrink the length parameter.
+
+        """
+
+        self.debug('Zeroing individual blocks')
+
+        # We first do a binary search on the hope that a lot of blocks are
+        # replacable. If not, we only pay a log(n) cost so it's no big deal.
+
+        # We can replace all blocks >= hi with zero. We cannot replace
+        # all blocks >= lo with zero.
+        lo = 0
+        hi = len(self.last_data.blocks)
+        while lo + 1 < hi:
+            mid = (lo + hi) // 2
+            try:
+                u = self.last_data.blocks[mid][0]
+            except IndexError:
+                # This shouldn't really happen, but may in the presence of a
+                # bad test function whose block structure varies based on some
+                # sort of external data. We could possibly detect this better
+                # and signal an error, but it's hard to do so reliably so
+                # instead we just try to be robust in the face of it.
+                break
+            if self.incorporate_new_buffer(
+                self.last_data.buffer[:u] +
+                hbytes(len(self.last_data.buffer) - u),
+            ):
+                hi = mid
+            else:
+                lo = mid
+
+        for i in hrange(len(self.last_data.blocks) - 1, -1, -1):
+            # The case where this is not true is hard to hit reliably, and only
+            # exists for similar reasons to the above: It guards against
+            # invalid data generation.
+            if i < len(self.last_data.blocks):  # pragma: no branch
+                u, v = self.last_data.blocks[i]
+                self.incorporate_new_buffer(
+                    self.last_data.buffer[:u] + hbytes(v - u) +
+                    self.last_data.buffer[v:],
+                )
+
+    def shrink(self):
+        # We assume that if an all-zero block of bytes is an interesting
+        # example then we're not going to do better than that.
+        # This might not technically be true: e.g. for integers() | booleans()
+        # the simplest example is actually [1, 0]. Missing this case is fairly
+        # harmless and this allows us to make various simplifying assumptions
+        # about the structure of the data (principally that we're never
+        # operating on a block of all zero bytes so can use non-zeroness as a
+        # signpost of complexity).
+        if (
+            not any(self.last_data.buffer) or
+            self.incorporate_new_buffer(hbytes(len(self.last_data.buffer)))
+        ):
+            self.exit_reason = ExitReason.finished
+            return
+
         change_counter = -1
 
         while self.changed > change_counter:
@@ -504,6 +663,13 @@ class ConjectureRunner(object):
                         i += k
                 k //= 2
 
+            self.zero_blocks()
+
+            minimize(
+                self.last_data.buffer, self.incorporate_new_buffer,
+                cautious=True, random=self.random,
+            )
+
             if change_counter != self.changed:
                 self.debug('Restarting')
                 continue
@@ -525,25 +691,6 @@ class ConjectureRunner(object):
                 self.incorporate_new_buffer(hbytes(buffer))
                 i += 1
 
-            self.debug('Replacing individual blocks with simpler blocks')
-            i = 0
-            while i < len(self.last_data.blocks):
-                u, v = self.last_data.blocks[i]
-                buf = self.last_data.buffer
-                block = buf[u:v]
-                n = v - u
-                all_blocks = sorted(set([hbytes(n)] + [
-                    buf[a:a + n]
-                    for a in self.last_data.block_starts[n]
-                ]))
-                better_blocks = all_blocks[:all_blocks.index(block)]
-                for b in better_blocks:
-                    if self.incorporate_new_buffer(
-                        buf[:u] + b + buf[v:]
-                    ):
-                        break
-                i += 1
-
             self.debug('Simultaneous shrinking of duplicated blocks')
             block_counter = -1
             while block_counter < self.changed:
@@ -568,19 +715,13 @@ class ConjectureRunner(object):
                     minimize(
                         block,
                         lambda b: self.incorporate_new_buffer(replace(b)),
-                        self.random
+                        random=self.random,
                     )
 
             if change_counter != self.changed:
                 self.debug('Restarting')
                 continue
 
-            self.debug('Lexicographical minimization of whole buffer')
-            minimize(
-                self.last_data.buffer, self.incorporate_new_buffer,
-                cautious=True
-            )
-
             self.debug('Shrinking of individual blocks')
             i = 0
             while i < len(self.last_data.blocks):
@@ -590,7 +731,8 @@ class ConjectureRunner(object):
                     lambda b: self.incorporate_new_buffer(
                         self.last_data.buffer[:u] + b +
                         self.last_data.buffer[v:],
-                    ), self.random
+                    ),
+                    random=self.random,
                 )
                 i += 1
 
@@ -624,10 +766,6 @@ class ConjectureRunner(object):
                             break
                     i += 1
 
-            if change_counter != self.changed:
-                self.debug('Restarting')
-                continue
-
             self.debug('Shuffling suffixes while shrinking %r' % (
                 self.last_data.bind_points,
             ))
@@ -651,7 +789,8 @@ class ConjectureRunner(object):
                             return True
                     return False
                 minimize(
-                    self.last_data.buffer[:cutoff], test_value, cautious=True
+                    self.last_data.buffer[:cutoff], test_value, cautious=True,
+                    random=self.random,
                 )
                 b += 1
 
