@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2015 David R. MacIver
+# Most of this work is copyright (C) 2013-2017 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -29,7 +29,9 @@ class FlatMapStrategy(SearchStrategy):
         super(FlatMapStrategy, self).__init__()
         self.flatmapped_strategy = strategy
         self.expand = expand
-        self.is_empty = strategy.is_empty
+
+    def calc_is_empty(self):
+        return self.flatmapped_strategy.is_empty
 
     def __repr__(self):
         if not hasattr(self, u'_cached_repr'):
@@ -40,4 +42,12 @@ class FlatMapStrategy(SearchStrategy):
 
     def do_draw(self, data):
         source = data.draw(self.flatmapped_strategy)
+        data.mark_bind()
         return data.draw(self.expand(source))
+
+    @property
+    def branches(self):
+        return [
+            FlatMapStrategy(strategy=strategy, expand=self.expand)
+            for strategy in self.flatmapped_strategy.branches
+        ]
